'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _hoistNonReactStatics = require('hoist-non-react-statics');

var _hoistNonReactStatics2 = _interopRequireDefault(_hoistNonReactStatics);

var _MidiDeviceProvider = require('../MidiDeviceProvider');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var babelPluginFlowReactPropTypes_proptype_ComponentType = require('prop-types').func;

var wrapWithMidi = function wrapWithMidi(Base) {
  var _WithMidi$contextType;

  var componentName = Base.displayName || Base.name || 'Component';

  var WithMidi = function (_Component) {
    _inherits(WithMidi, _Component);

    function WithMidi() {
      _classCallCheck(this, WithMidi);

      var _this = _possibleConstructorReturn(this, _Component.call(this));

      _this.state = {};

      _this.setMidiInput = _this.setMidiInput.bind(_this);
      _this.setMidiOutput = _this.setMidiOutput.bind(_this);
      return _this;
    }

    WithMidi.prototype.componentWillMount = function componentWillMount() {
      var _this2 = this;

      var midiContext = this.context[_MidiDeviceProvider.CHANNEL];

      if (!midiContext) {
        console.warn('[withMidi] - Unable to get midi context. It seems you are not using a MidiDeviceProvider.');
        return;
      }

      this.unsubscribe = midiContext.subscribe(function (midi) {
        _this2.setState({ midi: midi });
      });
    };

    WithMidi.prototype.componentWillUnmount = function componentWillUnmount() {
      if (typeof this.unsubscribe === 'function') {
        this.unsubscribe();
      }
    };

    WithMidi.prototype.setMidiInput = function setMidiInput(id) {
      var midiContext = this.context[_MidiDeviceProvider.CHANNEL];

      if (!midiContext) {
        console.warn('[withMidi] - Unable to get midi context. It seems you are not using a MidiDeviceProvider.');
        return;
      }

      if (this.state.midi && Array.isArray(this.state.midi.inputs)) {
        midiContext.setMidiInput(this.state.midi.inputs.filter(function (i) {
          return i.id === id;
        })[0]);
      }
    };

    WithMidi.prototype.setMidiOutput = function setMidiOutput(id) {
      var midiContext = this.context[_MidiDeviceProvider.CHANNEL];

      if (!midiContext) {
        console.warn('[withMidi] - Unable to get midi context. It seems you are not using a MidiDeviceProvider.');
        return;
      }

      if (this.state.midi && Array.isArray(this.state.midi.outputs)) {
        midiContext.setMidiOutput(this.state.midi.outputs.filter(function (i) {
          return i.id === id;
        })[0]);
      }
    };

    WithMidi.prototype.render = function render() {
      return _react2.default.createElement(Base, _extends({}, this.props, {
        midi: this.state.midi,
        setMidiInput: this.setMidiInput,
        setMidiOutput: this.setMidiOutput
      }));
    };

    return WithMidi;
  }(_react.Component);

  WithMidi.contextTypes = (_WithMidi$contextType = {}, _WithMidi$contextType[_MidiDeviceProvider.CHANNEL] = _MidiDeviceProvider.CONTEXT_CHANNEL_SHAPE, _WithMidi$contextType);
  WithMidi.displayName = 'WithMidi(' + componentName + ')';


  return (0, _hoistNonReactStatics2.default)(WithMidi, Base);
};

wrapWithMidi.propTypes = babelPluginFlowReactPropTypes_proptype_ComponentType === require('prop-types').any ? {} : babelPluginFlowReactPropTypes_proptype_ComponentType;
exports.default = wrapWithMidi;
module.exports = exports['default'];